@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "../config.h"
+#include <config.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
@@ -25,7 +25,6 @@
 #include <unistd.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <gtk/gtk.h>
 #include "debug.h"
 #include "marshal.h"
 #include "reaper.h"
@@ -115,7 +114,7 @@ int
 vte_reaper_add_child(GPid pid)
 {
 #if GLIB_CHECK_VERSION(2,4,0)
-	return g_child_watch_add_full(G_PRIORITY_HIGH,
+	return g_child_watch_add_full(G_PRIORITY_LOW,
 				      pid,
 				      vte_reaper_child_watch_cb,
 				      vte_reaper_get(),
@@ -270,7 +269,7 @@ main(int argc, char **argv)
 	VteReaper *reaper;
 	pid_t p, q;
 
-	_vte_debug_parse_string(getenv("VTE_DEBUG_FLAGS"));
+	_vte_debug_init();
 
 	g_type_init();
 	context = g_main_context_default();
