@@ -30,7 +30,7 @@
 G_BEGIN_DECLS
 
 #define VTE_DRAW_SINGLE_WIDE_CHARACTERS	"ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
-					"abcdefgjijklmnopqrstuvwxyz" \
+					"abcdefghijklmnopqrstuvwxyz" \
 					"0123456789./+@&"
 #define VTE_DRAW_DOUBLE_WIDE_CHARACTERS 0x4e00, 0x4e8c, 0x4e09, 0x56db, 0x4e94,\
 					0xac00, 0xac01, 0xac04, 0xac08, 0xac10
@@ -62,30 +62,23 @@ struct _vte_draw_impl {
 	GdkColormap* (*get_colormap)(struct _vte_draw *draw);
 	void (*start)(struct _vte_draw *draw);
 	void (*end)(struct _vte_draw *draw);
-	void (*set_background_color)(struct _vte_draw *, GdkColor *, guint16);
 	void (*set_background_image)(struct _vte_draw *,
 				     enum VteBgSourceType type,
 				     GdkPixbuf *pixbuf,
 				     const char *file,
 				     const GdkColor *color,
 				     double saturation);
-	gboolean requires_repaint;
 	void (*clip)(struct _vte_draw *, GdkRegion *);
+	gboolean always_requires_clear;
 	void (*clear)(struct _vte_draw *, gint, gint, gint, gint);
 	void (*set_text_font)(struct _vte_draw *,
 			      const PangoFontDescription *,
 			      VteTerminalAntiAlias);
-	int (*get_text_width)(struct _vte_draw *);
-	int (*get_text_height)(struct _vte_draw *);
-	int (*get_text_ascent)(struct _vte_draw *);
+	void (*get_text_metrics)(struct _vte_draw *, gint *, gint *, gint *);
 	int (*get_char_width)(struct _vte_draw *, gunichar c, int columns);
-	gboolean (*get_using_fontconfig)(struct _vte_draw *);
 	void (*draw_text)(struct _vte_draw *,
 			  struct _vte_draw_text_request *, gsize,
 			  GdkColor *, guchar);
-	gboolean (*draw_char)(struct _vte_draw *,
-			      struct _vte_draw_text_request *,
-			      GdkColor *, guchar);
 	gboolean (*has_char)(struct _vte_draw *, gunichar);
 	void (*draw_rectangle)(struct _vte_draw *,
 			       gint, gint, gint, gint,
@@ -93,14 +86,24 @@ struct _vte_draw_impl {
 	void (*fill_rectangle)(struct _vte_draw *,
 			       gint, gint, gint, gint,
 			       GdkColor *, guchar);
-	void (*set_scroll)(struct _vte_draw *, gint, gint);
 };
 
 struct _vte_draw {
 	GtkWidget *widget;
+
 	gboolean started;
-	gint width, height, ascent;
+
+	guint16 bg_opacity;
+	GdkColor bg_color;
+	enum VteBgSourceType bg_type;
+
+	gint scrollx, scrolly;
+
+	gboolean requires_clear;
+
 	const struct _vte_draw_impl *impl;
+
+	/* for use by impl */
 	gpointer impl_data;
 };
 
@@ -122,16 +125,17 @@ void _vte_draw_end(struct _vte_draw *draw);
 
 /* Set the background color, a background pixbuf (if you want transparency,
    you'll have to do that yourself), and clear an area to the default. */
+void _vte_draw_set_background_opacity(struct _vte_draw *draw,
+				      guint16 opacity);
 void _vte_draw_set_background_color(struct _vte_draw *draw,
-				    GdkColor *color,
-				    guint16 opacity);
+				    GdkColor *color);
 void _vte_draw_set_background_image(struct _vte_draw *draw,
 				    enum VteBgSourceType type,
 				    GdkPixbuf *pixbuf,
 				    const char *file,
 				    const GdkColor *color,
 				    double saturation);
-gboolean _vte_draw_requires_repaint(struct _vte_draw *draw);
+gboolean _vte_draw_requires_clear (struct _vte_draw *draw);
 gboolean _vte_draw_clip(struct _vte_draw *draw, GdkRegion *region);
 void _vte_draw_clear(struct _vte_draw *draw,
 		     gint x, gint y, gint width, gint height);
@@ -141,11 +145,9 @@ void _vte_draw_set_text_font(struct _vte_draw *draw,
 			     const PangoFontDescription *fontdesc,
 			     VteTerminalAntiAlias anti_alias);
 /* Read font metrics. */
-int _vte_draw_get_text_width(struct _vte_draw *draw);
-int _vte_draw_get_text_height(struct _vte_draw *draw);
-int _vte_draw_get_text_ascent(struct _vte_draw *draw);
+void _vte_draw_get_text_metrics(struct _vte_draw *draw,
+				gint *width, gint *height, gint *ascent);
 int _vte_draw_get_char_width(struct _vte_draw *draw, gunichar c, int columns);
-gboolean _vte_draw_get_using_fontconfig(struct _vte_draw *draw);
 
 /* Draw text or rectangles. */
 void _vte_draw_text(struct _vte_draw *draw,
